@@ -24,65 +24,35 @@ import os
 import random
 import shlex
 import signal
+import subprocess
 import sys
 import time
 
 import enum
 from oslo_utils import encodeutils
-from oslo_utils import importutils
 from oslo_utils import strutils
 from oslo_utils import timeutils
 
 from oslo_concurrency._i18n import _
 
 
-# NOTE(bnemec): eventlet doesn't monkey patch subprocess, so we need to
-# determine the proper subprocess module to use ourselves.  I'm using the
-# time module as the check because that's a monkey patched module we use
-# in combination with subprocess below, so they need to match.
-eventlet = importutils.try_import('eventlet')
-eventlet_patched = eventlet and eventlet.patcher.is_monkey_patched(time)
-if eventlet_patched:
-    if os.name == 'nt':
-        # subprocess.Popen.communicate will spawn two threads consuming
-        # stdout/stderr when passing data through stdin. We need to make
-        # sure that *native* threads will be used as pipes are blocking
-        # on Windows.
-        # Recent eventlet versions actually do patch subprocess.
-        subprocess = eventlet.patcher.original('subprocess')
-        subprocess.threading = eventlet.patcher.original('threading')
-    else:
-        from eventlet.green import subprocess
-
-    from eventlet import tpool
-
-    # Monkey patch the original current_thread to use the up-to-date _active
-    # global variable. See https://bugs.launchpad.net/bugs/1863021 and
-    # https://github.com/eventlet/eventlet/issues/592
-    import __original_module_threading as orig_threading
-    import threading
-    orig_threading.current_thread.__globals__['_active'] = threading._active
-else:
-    import subprocess
-
-
 LOG = logging.getLogger(__name__)
 
 
 class InvalidArgumentError(Exception):
     def __init__(self, message=None):
-        super(InvalidArgumentError, self).__init__(message)
+        super().__init__(message)
 
 
 class UnknownArgumentError(Exception):
     def __init__(self, message=None):
-        super(UnknownArgumentError, self).__init__(message)
+        super().__init__(message)
 
 
 class ProcessExecutionError(Exception):
     def __init__(self, stdout=None, stderr=None, exit_code=None, cmd=None,
                  description=None):
-        super(ProcessExecutionError, self).__init__(
+        super().__init__(
             stdout, stderr, exit_code, cmd, description)
         self.exit_code = exit_code
         self.stderr = stderr
@@ -113,7 +83,7 @@ class ProcessExecutionError(Exception):
 
 class NoRootWrapSpecified(Exception):
     def __init__(self, message=None):
-        super(NoRootWrapSpecified, self).__init__(message)
+        super().__init__(message)
 
 
 def _subprocess_setup(on_preexec_fn):
@@ -147,7 +117,7 @@ LOG_FINAL_ERROR = LogErrors.FINAL
 LOG_DEFAULT_ERROR = LogErrors.DEFAULT
 
 
-class ProcessLimits(object):
+class ProcessLimits:
     """Resource limits on a process.
 
     Attributes:
@@ -193,7 +163,7 @@ class ProcessLimits(object):
         for limit in self._LIMITS:
             val = getattr(self, limit)
             if val is not None:
-                args.append("%s=%s" % (self._LIMITS[limit], val))
+                args.append("{}={}".format(self._LIMITS[limit], val))
         return args
 
 
@@ -269,8 +239,8 @@ def execute(*cmd, **kwargs):
                             below for a detailed description.
     :type prlimit:          :class:`ProcessLimits`
     :param python_exec:     The python executable to use for enforcing
-                            prlimits. If this is not set it will default to use
-                            sys.executable.
+                            prlimits. If this is not set or is None, it will
+                            default to use sys.executable.
     :type python_exec:      string
     :param timeout:         Timeout (in seconds) to wait for the process
                             termination. If timeout is reached,
@@ -329,7 +299,7 @@ def execute(*cmd, **kwargs):
     on_completion = kwargs.pop('on_completion', None)
     preexec_fn = kwargs.pop('preexec_fn', None)
     prlimit = kwargs.pop('prlimit', None)
-    python_exec = kwargs.pop('python_exec', sys.executable)
+    python_exec = kwargs.pop('python_exec', None) or sys.executable
     timeout = kwargs.pop('timeout', None)
 
     if isinstance(check_exit_code, bool):
@@ -362,16 +332,11 @@ def execute(*cmd, **kwargs):
     cmd = [str(c) for c in cmd]
 
     if prlimit:
-        if os.name == 'nt':
-            LOG.log(loglevel,
-                    _('Process resource limits are ignored as '
-                      'this feature is not supported on Windows.'))
-        else:
-            args = [python_exec, '-m', 'oslo_concurrency.prlimit']
-            args.extend(prlimit.prlimit_args())
-            args.append('--')
-            args.extend(cmd)
-            cmd = args
+        args = [python_exec, '-m', 'oslo_concurrency.prlimit']
+        args.extend(prlimit.prlimit_args())
+        args.append('--')
+        args.extend(cmd)
+        cmd = args
 
     sanitized_cmd = strutils.mask_password(' '.join(cmd))
 
@@ -384,38 +349,23 @@ def execute(*cmd, **kwargs):
             LOG.log(loglevel, _('Running cmd (subprocess): %s'), sanitized_cmd)
             _PIPE = subprocess.PIPE  # pylint: disable=E1101
 
-            if os.name == 'nt':
-                on_preexec_fn = None
-                close_fds = False
-            else:
-                on_preexec_fn = functools.partial(_subprocess_setup,
-                                                  preexec_fn)
-                close_fds = True
+            on_preexec_fn = functools.partial(_subprocess_setup, preexec_fn)
 
             obj = subprocess.Popen(cmd,
                                    stdin=_PIPE,
                                    stdout=_PIPE,
                                    stderr=_PIPE,
-                                   close_fds=close_fds,
+                                   close_fds=True,
                                    preexec_fn=on_preexec_fn,
-                                   shell=shell,  # nosec:B604
+                                   shell=shell,
                                    cwd=cwd,
-                                   env=env_variables)
+                                   env=env_variables)  # nosec:B602
 
             if on_execute:
                 on_execute(obj)
 
             try:
-                # eventlet.green.subprocess is not really greenthread friendly
-                # on Windows. In order to avoid blocking other greenthreads,
-                # we have to wrap this call using tpool.
-                if eventlet_patched and os.name == 'nt':
-                    result = tpool.execute(obj.communicate,
-                                           process_input,
-                                           timeout=timeout)
-                else:
-                    result = obj.communicate(process_input,
-                                             timeout=timeout)
+                result = obj.communicate(process_input, timeout=timeout)
 
                 obj.stdin.close()  # pylint: disable=E1101
                 _returncode = obj.returncode  # pylint: disable=E1101
