@@ -14,6 +14,7 @@
 
 import collections
 import errno
+import fcntl
 import multiprocessing
 import os
 import signal
@@ -24,32 +25,12 @@ import threading
 import time
 from unittest import mock
 
-from oslo_config import cfg
 from oslotest import base as test_base
 
 from oslo_concurrency.fixture import lockutils as fixtures
 from oslo_concurrency import lockutils
 from oslo_config import fixture as config
 
-if sys.platform == 'win32':
-    import msvcrt
-else:
-    import fcntl
-
-
-def lock_file(handle):
-    if sys.platform == 'win32':
-        msvcrt.locking(handle.fileno(), msvcrt.LK_NBLCK, 1)
-    else:
-        fcntl.flock(handle, fcntl.LOCK_EX | fcntl.LOCK_NB)
-
-
-def unlock_file(handle):
-    if sys.platform == 'win32':
-        msvcrt.locking(handle.fileno(), msvcrt.LK_UNLCK, 1)
-    else:
-        fcntl.flock(handle, fcntl.LOCK_UN)
-
 
 def lock_files(handles_dir, out_queue):
     with lockutils.lock('external', 'test-', external=True):
@@ -66,10 +47,10 @@ def lock_files(handles_dir, out_queue):
         count = 0
         for handle in handles:
             try:
-                lock_file(handle)
+                fcntl.flock(handle, fcntl.LOCK_EX | fcntl.LOCK_NB)
                 count += 1
-                unlock_file(handle)
-            except IOError:
+                fcntl.flock(handle, fcntl.LOCK_UN)
+            except OSError:
                 os._exit(2)
             finally:
                 handle.close()
@@ -79,7 +60,7 @@ def lock_files(handles_dir, out_queue):
 class LockTestCase(test_base.BaseTestCase):
 
     def setUp(self):
-        super(LockTestCase, self).setUp()
+        super().setUp()
         self.config = self.useFixture(config.Config(lockutils.CONF)).config
 
     def test_synchronized_wrapped_function_metadata(self):
@@ -186,6 +167,13 @@ class LockTestCase(test_base.BaseTestCase):
                 pass
         self.assertRaises(NotImplementedError, do_test)
 
+    def test_fair_lock_with_nonblocking(self):
+        def do_test():
+            with lockutils.lock('testlock', 'test-', fair=True,
+                                blocking=False):
+                pass
+        self.assertRaises(NotImplementedError, do_test)
+
     def test_nested_synchronized_external_works(self):
         """We can nest external syncs."""
         self.config(lock_path=tempfile.mkdtemp(), group='oslo_concurrency')
@@ -354,22 +342,10 @@ class LockTestCase(test_base.BaseTestCase):
         with lockutils.lock("foobar"):
             pass
 
-    def test_deprecated_names(self):
-        paths = self.create_tempfiles([['fake.conf', '\n'.join([
-            '[DEFAULT]',
-            'lock_path=foo',
-            'disable_process_locking=True'])
-        ]])
-        conf = cfg.ConfigOpts()
-        conf(['--config-file', paths[0]])
-        conf.register_opts(lockutils._opts, 'oslo_concurrency')
-        self.assertEqual('foo', conf.oslo_concurrency.lock_path)
-        self.assertTrue(conf.oslo_concurrency.disable_process_locking)
-
 
 class FileBasedLockingTestCase(test_base.BaseTestCase):
     def setUp(self):
-        super(FileBasedLockingTestCase, self).setUp()
+        super().setUp()
         self.lock_dir = tempfile.mkdtemp()
 
     def test_lock_file_exists(self):
@@ -405,7 +381,7 @@ class FileBasedLockingTestCase(test_base.BaseTestCase):
                     lock1.trylock()
                     lock1.unlock()
                     time.sleep(0)
-                except IOError:
+                except OSError:
                     # This is what we expect to happen
                     break
             else:
@@ -421,7 +397,7 @@ class FileBasedLockingTestCase(test_base.BaseTestCase):
                     try:
                         lock2.trylock()
                         have_lock = True
-                    except IOError:
+                    except OSError:
                         pass
             finally:
                 # NOTE(bnemec): This is racy, but I don't want to add any
@@ -430,6 +406,35 @@ class FileBasedLockingTestCase(test_base.BaseTestCase):
                 time.sleep(.5)
                 os._exit(0)
 
+    def test_interprocess_nonblocking_external_lock(self):
+        """Check that we're not actually blocking between processes."""
+
+        nb_calls = multiprocessing.Value('i', 0)
+
+        @lockutils.synchronized('foo', blocking=False, external=True,
+                                lock_path=self.lock_dir)
+        def foo(param):
+            """Simulate a long-running operation in a process."""
+            param.value += 1
+            time.sleep(.5)
+
+        def other(param):
+            foo(param)
+
+        process = multiprocessing.Process(target=other, args=(nb_calls, ))
+        process.start()
+        # Make sure the other process grabs the lock
+        start = time.time()
+        while not os.path.exists(os.path.join(self.lock_dir, 'foo')):
+            if time.time() - start > 5:
+                self.fail('Timed out waiting for process to grab lock')
+            time.sleep(0)
+        process1 = multiprocessing.Process(target=other, args=(nb_calls, ))
+        process1.start()
+        process1.join()
+        process.join()
+        self.assertEqual(1, nb_calls.value)
+
     def test_interthread_external_lock(self):
         call_list = []
 
@@ -465,6 +470,62 @@ class FileBasedLockingTestCase(test_base.BaseTestCase):
         thread.join()
         self.assertEqual(['other', 'other', 'main', 'main'], call_list)
 
+    def test_interthread_nonblocking_external_lock(self):
+        call_list = []
+
+        @lockutils.synchronized('foo', external=True, blocking=False,
+                                lock_path=self.lock_dir)
+        def foo(param):
+            """Simulate a long-running threaded operation."""
+            call_list.append(param)
+            time.sleep(.5)
+            call_list.append(param)
+
+        def other(param):
+            foo(param)
+
+        thread = threading.Thread(target=other, args=('other',))
+        thread.start()
+        # Make sure the other thread grabs the lock
+        start = time.time()
+        while not os.path.exists(os.path.join(self.lock_dir, 'foo')):
+            if time.time() - start > 5:
+                self.fail('Timed out waiting for thread to grab lock')
+            time.sleep(0)
+        thread1 = threading.Thread(target=other, args=('main',))
+        thread1.start()
+        thread1.join()
+        thread.join()
+        self.assertEqual(['other', 'other'], call_list)
+
+    def test_interthread_nonblocking_internal_lock(self):
+        call_list = []
+
+        @lockutils.synchronized('foo', blocking=False,
+                                lock_path=self.lock_dir)
+        def foo(param):
+            # Simulate a long-running threaded operation.
+            call_list.append(param)
+            time.sleep(.5)
+            call_list.append(param)
+
+        def other(param):
+            foo(param)
+
+        thread = threading.Thread(target=other, args=('other',))
+        thread.start()
+        # Make sure the other thread grabs the lock
+        start = time.time()
+        while not call_list:
+            if time.time() - start > 5:
+                self.fail('Timed out waiting for thread to grab lock')
+            time.sleep(0)
+        thread1 = threading.Thread(target=other, args=('main',))
+        thread1.start()
+        thread1.join()
+        thread.join()
+        self.assertEqual(['other', 'other'], call_list)
+
     def test_non_destructive(self):
         lock_file = os.path.join(self.lock_dir, 'not-destroyed')
         with open(lock_file, 'w') as f:
@@ -478,7 +539,7 @@ class FileBasedLockingTestCase(test_base.BaseTestCase):
 class LockutilsModuleTestCase(test_base.BaseTestCase):
 
     def setUp(self):
-        super(LockutilsModuleTestCase, self).setUp()
+        super().setUp()
         self.old_env = os.environ.get('OSLO_LOCK_PATH')
         if self.old_env is not None:
             del os.environ['OSLO_LOCK_PATH']
@@ -486,7 +547,7 @@ class LockutilsModuleTestCase(test_base.BaseTestCase):
     def tearDown(self):
         if self.old_env is not None:
             os.environ['OSLO_LOCK_PATH'] = self.old_env
-        super(LockutilsModuleTestCase, self).tearDown()
+        super().tearDown()
 
     def test_main(self):
         script = '\n'.join([
@@ -518,7 +579,7 @@ class LockutilsModuleTestCase(test_base.BaseTestCase):
 class TestLockFixture(test_base.BaseTestCase):
 
     def setUp(self):
-        super(TestLockFixture, self).setUp()
+        super().setUp()
         self.config = self.useFixture(config.Config(lockutils.CONF)).config
         self.tempdir = tempfile.mkdtemp()
 
@@ -527,7 +588,7 @@ class TestLockFixture(test_base.BaseTestCase):
 
     def tearDown(self):
         self._check_in_lock()
-        super(TestLockFixture, self).tearDown()
+        super().tearDown()
 
     def test_lock_fixture(self):
         # Setup lock fixture to test that teardown is inside the lock
@@ -540,7 +601,7 @@ class TestLockFixture(test_base.BaseTestCase):
 class TestGetLockPath(test_base.BaseTestCase):
 
     def setUp(self):
-        super(TestGetLockPath, self).setUp()
+        super().setUp()
         self.conf = self.useFixture(config.Config(lockutils.CONF)).conf
 
     def test_get_default(self):
